#!/bin/bash

# This script is executed inside pre_test_hook function in devstack gate.

# Install Murano devstack integration
source ./pre_test_hook_common.sh
MURANO_BASE=/opt/stack/new/murano/contrib/devstack

